# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 m@n <rmaneschi@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('showmyfaves')

from gi.repository import Gtk, Gdk, Unity # pylint: disable=E0611
import logging
log = logging.getLogger('showmyfaves')
logging.basicConfig(level=logging.INFO)

from showmyfaves_lib import Window
from showmyfaves.AboutShowmyfavesDialog import AboutShowmyfavesDialog
from showmyfaves.PreferencesShowmyfavesDialog import PreferencesShowmyfavesDialog
from LayoutBookmark import LayoutBookmark

from server.ShowmyfavesClient import Client
from server.ShowmyfavesServer import FakeServer
from server.ShowmyfavesFactoryCommand import FactoryCommand
from ScreenshotWebsite import ScreenshotWebsite
from LayoutBookmarkContainer import LayoutBookmarkContainer
from server.ShowmyfavesModel import Bookmark, Tag
from TagGraphic import TagGraphic
from TagFrame import TagFrame
from SortMenu import SortMenu
from ParserChooser import ParserChooser

# See showmyfaves_lib.Window.py for more details about how this class works
class ShowmyfavesWindow(Window):
    log = logging.getLogger('ShowmyfavesWindow')

    __gtype_name__ = "ShowmyfavesWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(ShowmyfavesWindow, self).finish_initializing(builder)
        self.AboutDialog = AboutShowmyfavesDialog
        self.PreferencesDialog = PreferencesShowmyfavesDialog
        
        ## Construction du menu de tri        
        self.menuToolButtonTri = self.builder.get_object("btnSortBookmarks")
        self.menu = SortMenu()
        self.menuToolButtonTri.set_menu(self.menu)
        
        ## Construction du menu de choix de web browser        
        self.menuToolButtonParser = self.builder.get_object("btnChooseParser")
        self.menuParser = ParserChooser()
        self.menuToolButtonParser.set_menu(self.menuParser)
        
        ## Variables de classe
        self.nbBookmarks = 0
        self.nbTotalBookmarks = 0        
        
        ## Variables graphiques :
        self.toolbar = builder.get_object("toolbar")
        self.btnAddBookmark = builder.get_object("btnAddBookmark")
        self.vboxCenter = builder.get_object("vbox-center")
        
        ## Style
        contextToolbar = self.toolbar.get_style_context()
        contextToolbar.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)                
        contextWindow = self.get_style_context()
        contextWindow.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        
        self.layoutBookmarkContainer = LayoutBookmarkContainer()
        self.vboxCenter.pack_start(self.layoutBookmarkContainer,True,  True,10)        

        boxEtiquetteMenu = self.builder.get_object("boxEtiquetteMenu")
        self.tagFrame = TagFrame()
        boxEtiquetteMenu.add(self.tagFrame)

        self.searchEntry = self.builder.get_object("searchEntry")
        self.searchButton = self.builder.get_object("searchButton")

        self.row = 0
        self.line = 0
        
        self.nbBookmarks = 0
        self.nbTotalBookmarks = 0
        
        ## connexion avec le client
        self.client = Client(self)
        self.client.sendCommand(FactoryCommand.makeGetTags())
        self.client.sendCommand(FactoryCommand.makeGetNbBookmarks())
        self.client.sendCommand(FactoryCommand.makeGetBookmarks())
        
    def on_searchButton_clicked(self, widget):
        self.cleanChildren()
        self.client.sendCommand(FactoryCommand.makeGetNbBookmarks())
        self.client.sendCommand(FactoryCommand.makeGetBookmarks())
        
    def on_searchEntry_activate(self, widget):
        self.cleanChildren()
        FactoryCommand.INSTANCE().filters = [self.searchEntry.get_text()]
        self.client.sendCommand(FactoryCommand.makeGetNbBookmarks())
        self.client.sendCommand(FactoryCommand.makeGetBookmarks())
        
    def on_btnAddBookmark_clicked(self, widget):
        pass
                
    def on_destroy(self, widget, data=None):
        self.client.stop()
        super(ShowmyfavesWindow, self).on_destroy(widget, data)

    def cleanChildren(self):
        self.layoutBookmarkContainer.cleanChildren()
        
    def error(self, errorMessage):
        # lancer notification par exemple pour afficher cette erreeur
        self.log.info('error %s', errorMessage)
         
    def setBookmarks(self, listBookmarks):
        self.nbBookmarks += len(listBookmarks)
        self.builder.get_object("labelStatusArea").set_text("Nb bookmarks : "+str(self.nbBookmarks)+"/"+str(self.nbTotalBookmarks))
        self.layoutBookmarkContainer.addLayoutbookmark(listBookmarks)
                
    def setNbBookmarks(self, nb):
        self.nbBookmarks = 0
        self.nbTotalBookmarks = nb
        self.builder.get_object("labelStatusArea").set_text("Nb bookmarks : "+str(nb))
    
    def setTags(self, listTags):
        self.tagFrame.addBookmarkTags(listTags)
        
    def setParsers(self, nomParsersDisponibles):
        # permet de faire la configuration
        self.grille.attach(LayoutBookmark("data/media/fav2.png", "test2"), 0, 1, 1, 1)
        self.log.info('setParsers %s', nomParsersDisponibles)
        
    def setSorts(self, nomSortsDisponibles):
        # affiche la liste des tris disponibles
        self.grille.attach(LayoutBookmark("data/media/fav2.png", "test2"), 0, 1, 1, 1)
        self.log.info('setSorts %s', nomSortsDisponibles)
