from __future__ import print_function

import asyncore
import collections
import logging
import socket
from threading import Thread
import time
import json
import gobject

import ShowmyfavesUtil
import ShowmyfavesModel

class Client(asyncore.dispatcher):

    log = logging.getLogger('Client')
    INSTANCE = None

    '''listner must have methods : 
            - error(string)
            - setBookmarks(list<Bookmark>)
            - setParsers(list<String>)
            - setSorts(list<String>)
    '''
    def __init__(self, listner=None):
        asyncore.dispatcher.__init__(self)
        self.config = ShowmyfavesUtil.ConfigLoader.load()
        self.outbox = collections.deque()
        self.listner = listner
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.connect((self.config.HOST_URL, self.config.HOST_PORT))
        self.log.info('Connecting to host at %s', self.config.HOST_PORT)
        self.accuData = ""
        self.launchLoop()
        Client.INSTANCE = self
    
    #THREAD PART    
    def stop(self):
        self.close()

    def launchLoop(self):
        self.start1loop()
        gobject.timeout_add(200, self.launchLoop)
        
    def start1loop(self):
        asyncore.loop(timeout=0.1, count=1)
        return False
        
    #ASYCORE PART
    def handle_connect(self):
        self.log.info('Connecting to host')

    def handle_expt(self):
        self.log.info('Not connecting to host')
        self.stop()
        
    def handle_close(self):
        self.log.info('Handle close')
        self.stop()

    def sendCommand(self, commande):
        self.outbox.append(commande.toString())

    def writable(self):
        return len(self.outbox) > 0

    def handle_write(self):
        message = self.outbox.popleft()
        if len(message) > self.config.MAX_MESSAGE_LENGTH:
            raise ValueError('Message too long')
        Thread(target=self.send, args=(message,)).start()

    def handle_read(self):
        message = self.socket.recv(self.config.MAX_MESSAGE_LENGTH)
        self.accuData += message
        self.cutCommands()
        
    #SHOWMYFAVES PART
    def cutCommands(self):
        #end of command
        index = self.accuData.find("#||;")
        if index != -1:
            self.decryptReceivedData(self.accuData[:index])
            self.accuData = self.accuData[index+4:]
            self.cutCommands()
        
    def decryptReceivedData(self, message):
        try:
            data = json.loads(message)
            if self.listner != None:
                if "error" in data:
                    gobject.idle_add(self.listner.error, data["error"])
                else:
                    if data["cmd"] == "getBookmarks" and data["id"]+1 == ShowmyfavesModel.Command.generalId:
                        jsonBookmark = data["data"]
                        d = ShowmyfavesModel.Bookmark().fromDict(**jsonBookmark)
                        gobject.idle_add(self.listner.setBookmarks, [d])
                    elif data["cmd"] == "getNbBookmarks":
                        self.log.info("getNbBookmarks client = "+str(data["data"]))
                        gobject.idle_add(self.listner.setNbBookmarks, data["data"])
                    elif data["cmd"] == "getTags":
                        tag = data["data"]
                        t = ShowmyfavesModel.Tag.getTag(tag["type"], tag["name"])
                        gobject.idle_add(self.listner.setTags, [t])
                    elif data["cmd"] == "getParsers":
                        gobject.idle_add(self.listner.setParsers, data["data"])
                    elif data["cmd"] == "getSorts":
                        gobject.idle_add(self.listner.setSorts, data["data"])
            elif not data["id"] == ShowmyfavesModel.Command.generalId:
                self.log.info("Command id is out of date : "+str(data["id"]))
            else:
                self.log.info("No listner on client !!")
        except ValueError as e:
            self.log.error(message)

if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    client = Client()
    threadingClient = ThreadingClient()
    threadingClient.start()
    client.sendCommand('{"cmd":"getBookmarks","params":{"parser":["firefox_2"],"sort":["name"], "reverseSort":0}}')
    cmd = ""
    while cmd != "close":
        cmd = raw_input()
        client.sendCommand('{"cmd":"'+cmd+'"}')
    client.close()
    threadingClient.stop() 
	
