import json
import logging
import random

class Bookmark():
    
    log = logging.getLogger("bookmark")
    
    def __init__(self, name="", url="", addedDate="", description="", tags=""):
        self.url = url
        self.name = name
        self.addedDate = addedDate
        self.description = description
        self.tags = tags
            
    def fromDict(self, **entries):
        self.__dict__.update(entries)
        self.tags = []
        for tag in entries["tags"]:
            oTag = Tag()
            oTag.fromDict(tag)
            oTag = Tag.getTag(oTag.type, oTag.name)
            self.tags.append(oTag)
        return self
            
    def __str__(self):
        tags = ""
        return "Bookmark : name=" + self.name + " : url=" + self.url + " : description" + self.description + " : tags=" + tags + " : addedDate=" + self.addedDate
        
class Tag():
    
    log = logging.getLogger("Tag")
    INSTANCES = {}
    
    def __init__(self, tagType="", name=""):
        self.type = tagType
        self.name = name
        colorInt = random.randint(0, 16777215)
        self.color = "#%x" % colorInt
        
    def fromDict(self, entries):
        self.__dict__.update(entries)
    
    @staticmethod
    def getTag(tagType, name):
        if  not tagType+name in Tag.INSTANCES:
            tag = Tag(tagType, name)
            Tag.INSTANCES[tagType+name] = tag
        return Tag.INSTANCES[tagType+name]

class Command():

    generalId = 0

    def __init__(self, name, parsers=None, tags=None, filters=None, sort=None, reverseSort=None):
        self.name = name
        self.parsers = parsers
        self.tags = tags
        self.filters = filters
        self.sort = sort
        self.reverseSort = reverseSort
        self.id = Command.generalId
        if name == "getBookmarks":
            Command.generalId = Command.generalId + 1

    def toString(self):
        res = '{"cmd":"'+self.name+'","id":'+str(self.id)
        if not self.parsers == None or not self.tags == None or not self.filters == None or not self.sort == None:
            res = res + ',"params":{'
            #Parsers
            if not self.parsers == None:
                strParsers = "["
                for parser in self.parsers:
                    strParsers = strParsers + '"' + parser + '"' + ","
                if len(strParsers) > 1:
                    strParsers = strParsers[:-1]
                strParsers = strParsers + "]"
                res = res + '"parser":'+strParsers
            #tags
            if not self.tags == None:
                strTags = "["
                for t in self.tags:
                    strTags = strTags + '"' + t + '"' + ","
                if len(strTags) > 1:
                    strTags = strTags[:-1]
                strTags = strTags + "]"
                res = res + ',"tags":'+strTags
            #filter
            if not self.filters == None:
                strFilters = "["
                for f in self.filters:
                    strFilters = strFilters + '"' + f + '"' + ","
                if len(strFilters) > 1:
                    strFilters = strFilters[:-1]
                strFilters = strFilters + "]"
                res = res + ',"filter":'+strFilters
            if not self.sort == None:
                res = res +',"sort":["'+str(self.sort)+'"], "reverseSort":'+str(self.reverseSort)
            res = res + '}'
        res = res + '}'
        return res

class Configuration():
    def __init__(self, **entries):
        self.__dict__.update(entries)
        
    def __str__(self):
        return "Configuration : url=" + self.HOST_URL + " | port=" + str(self.HOST_PORT) + " | msgMAxLength=" + str(self.MAX_MESSAGE_LENGTH)

if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    log = logging.getLogger("mainModel")
    cmd = Command("getBookmarks", [], ["firefox"], ["salut","autre"], "date", 1)
    log.info(cmd.toString())
    cmd = Command("getParsers")
    log.info(cmd.toString())
    cmd = Command("getSorts")
    log.info(cmd.toString())
    cmd = Command("getNbBookmarks", [])
    log.info(cmd.toString())
